
--[=[
latest uses a assertion mechanism inspired by Hamcrest, a library of
matchers for building test expression.

in latest the term `predicates' is used instead of `matchers', and
by taking advantage the power of Lua -- function as first class value,
and functions can return multiple value.

so there is no assertions like

  asserteq(expected, actual)

or

  assertgreaterthan(limit, actual)

but instead, latest use assertions in the form of

  assertthat(value, predicate)

you may use your own predicates

the predicate is a function (or other callable value) which takes
a value as its only argument. it shall return true plus a description
or false plus a message, depending on if the predicate succeed or fails.

the second return value can be omitted in both situation, but you are
discouraged to do so, since latest may use that message to report.
]=]

-- custom predicates or _factory_
local even, haskey, arraycontains

test("demo the assertion and predicates concepts", function ()
   local result = { 2+2, "hello", flag = true }
   assertthat(result, isa("table"))
   assertthat(result[1], equalto(4))
   assertthat(#result, greaterthan(1))
   -- custom predicates follows
   assertthat(result[1], even)
   assertthat(result, haskey("flag"))
   assertthat(result, arraycontains("hello"))
end)

even = function (value)
   if value % 2 == 0 then
      return true, "an even number(" .. tostring(value) .. ")"
   else
      return false, "not an even number(" .. tostring(vale) .. ")"
   end
end

haskey = function (key)
   return function (value)
      if value[key] ~= nil then
         return true, "has the key " .. tostring(key)
      else
         return false, "doesn't have the key " .. tostring(key)
      end
   end
end

arraycontains = function (elem)
   return function (value)
      for i = 1, #value do
         local v = value[i]
         if v == elem then
            return true
         end
      end
      return false, "doesn't contains the element " .. tostring(elem)
   end
end

