
local modname = (...)
local m = require(modname .. ".core")
package.loaded[modname] = m
local predicates = require(modname .. ".predicates")

function m.report (stat)
   stat = stat or m.statistic()
   m.reportsummary(stat)
   if stat.grouprun ~= stat.grouppassed then
      m.reportfailed(stat)
   end
end

function m.reportsimple (stat)
   stat = stat or m.statistic()
   print  "====================="
   print(
      stat.grouprun == stat.grouppassed
      and "!!   TEST PASSED   !!"
      or  "!!   TEST FAILED   !!"
   )
   print  "====================="
end

function m.reportsummary (stat)
   stat = stat or m.statistic()
   print  "!!==========================================================="
   print  "!! SUMMARY :"
   print(
      stat.grouprun == stat.grouppassed
      and "!!   TEST PASSED"
      or  "!!   TEST FAILED"
   )
   print  "!!-----------------------------------------------------------"
   print  "!! GROUP statistics : "
   print(("!!   %d groups run, %d passed, %d failed, %d with error(s)."):format(stat.grouprun, stat.grouppassed, stat.groupfailed, stat.grouperror))
   print(("!!   (%d ignored, %d disabled.)"):format(stat.groupignored, stat.groupdisabled))
   print  "!!-----------------------------------------------------------"
   print  "!! TEST statistics : "
   print(("!!   %d tests run, %d passed, %d failed, %d with error(s)."):format(stat.run, stat.passed, stat.failed, stat.error))
   print(("!!   (%d ignored, %d disabled.)"):format(stat.ignored, stat.disabled))
end

function m.reportfailed (stat)
   stat = stat or m.statatistic()
   print "!!==========================================================="
   print "!! FAILED group(s) and test(s) : "
   for _i, group in m.allgroups() do
      if group.status == "failed" or group.status == "error" then
         print "!!-----------------------------------------------------------"
         print("!! group  : <<" .. group.name .. ">>")
         print("!! status : " .. group.status)
         for _name, test in pairs(group.tests) do
            if test.status == "failed" or test.status == "error" then
               print("!! ****************************************")
               print("!! test     : <" .. test.name .. ">")
               print("!!   status : " .. test.status)
               print("!!   detail : " .. test.detail) 
            end
         end
      end
   end
end

local loaded = {}
function m.loadtest (filename, groupname)
   if groupname ~= nil then
      m.grouppush(groupname)
   end
   local runall, report = m.runall, m.report
   local noop = function () end
   m.runall = noop
   m.report = noop
   if not loaded[filename] and pcall(dofile, filename) then
      loaded[filename] = true
   end
   m.runall, m.report = runall, report
   if groupname ~= nil then
      m.grouppop()
   end
end


_G.group = m.group
_G.setup = m.setup
_G.teardown = m.teardown
_G.test = m.test
_G.loadtest = m.loadtest
_G.failbecause = m.failbecause
_G.assertthat = m.assert

for name, value in pairs(predicates) do
   _G[name] = value
end

