
local predicates = {}
package.loaded[...] = predicates

function predicates.isa (expectedtype)
   return function (value)
      local t = type(value)
      return t == expectedtype, ("expected type %s(actual %s)"):format(expectedtype, t)
   end
end

function predicates.notnil (value)
   return value ~= nil, "expected a non-nil value"
end

function predicates.equalto (expected)
   return function (actual)
      return actual == expected, ("expected equal to %q(actual %q)"):format(tostring(expected), tostring(actual))
   end
end

function predicates.greaterthan (limit)
   return function (value)
      return value > limit, ("expected greater than %d(actual %d)"):format(limit, value)
   end
end

function predicates.lessthan (limit)
   return function (value)
      return value < limit, ("expected less than %d(actual %d)"):format(limit, value)
   end
end

function predicates.beginwith (substr)
   local expectation = ("expected string begin with %q"):format(substr)
   return function (actual)
      if type(actual) ~= "string" then
         return false, expectation .. "(not a string)"
      end
      return actual:sub(1, #substr) == substr, expectation
   end
end

function predicates.endwith (substr)
   local expectation = ("expected string end with %q"):format(substr)
   return function (actual)
      if type(actual) ~= "string" then
         return false, expectation .. "(not a string)"
      end
      return actual:sub(-#substr, -1) == substr, expectation
   end
end

function predicates.match (pattern, plain)
   local expectation = ("expected string matches pattern %q"):format(pattern)
   return function (actual)
      if type(actual) ~= "string" then
         return false, expectation .. "(not a string)"
      end
      return actual:find(pattern, 1, plain), expectation
   end
end

